{: This unit contains the TJPMColorComboBox which is a color selection combo-box
 which is independant of any third-party components.<p>
}
{Release notes
2-1-2000 I started this by making altering the jclrcbo component
2-2-2000 I adjusted the ListBox code to display the items at design-time
     and it fixes a problem with dynamcially-created controls
}
unit JPMColorListBox;

interface

{$I JPM.INC}

uses Classes, Controls,
     Graphics, JPMColorListObject, Messages,
     Stdctrls, Windows;

type
     TJPMColorListBox = class (TCustomListBox)
     private
          { Private fields of TJPMColorListBox }
          FColorSelObject: TJPMColorListObject;
          { Private methods of TJPMColorListBox }
          { Method to set variable and property values and create objects }
          procedure AutoInitialize;
          { Method to free any objects created by AutoInitialize }
          procedure AutoDestroy;
          { Read method for property SelectColor }
          function GetSelectColor: TColor;
          { Write method for property SelectColor }
          procedure SetSelectColor (Value: TColor);
          { Read method for property ShowSystemColors }
          function GetShowSystemColors: Boolean;
          { Write method for property AdditionalColors}
          procedure SetShowAdditionalColors (Value: Boolean);
          { Read method for property AdditionalColors}
          function GetShowAdditionalColors: Boolean;
          { Write method for property ShowSystemColors }
          procedure SetShowSystemColors (Value: Boolean);
          { Write method for property ShowCustomColor }
          procedure SetShowCustomColor (Value: Boolean);
          { Read method for property ShowCustomColor }
          function GetShowCustomColor: Boolean;
          {PreviewBoxWidth write method for PreviewBoxWidth }
          procedure SetPreviewBoxWidth (Value: Integer);
          {PreviewBoxWidth read method for PreviewBoxWidth }
          function GetPreviewBoxWidth: Integer;
          function GetShowBox: Boolean;
          function GetShowBoxBorder: Boolean;
          procedure SetShowBox (const Value: Boolean);
          procedure SetShowBoxBorder (const Value: Boolean);
          function GetTextOnColorBackground: Boolean;
          procedure SetTextOnColorBackground (const Value: Boolean);
          function GetShowStyledText: Boolean;
          procedure SetShowStyledText (const Value: Boolean);
     protected
          { Protected fields of TJPMColorListBox }
          procedure SetParent (AParent: TWinControl); override;
          procedure Click; override;
          procedure KeyPress (var Key: Char); override;
          { //Set Up color selection for combo-box }
          procedure Loaded; override;
          procedure PopulateComboBox; virtual;
          function GetJPMVersion: string;
          procedure SetJPMVersion (const Value: string);
     public
          { Public methods of TJPMColorListBox }
          constructor Create (AOwner: TComponent); override;
          destructor Destroy; override;
          {: override to display colors in box }
          procedure DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
          {: method for calling ColorSelection Dialog-box }
          function ExecuteDialog: Boolean;
     published
          { Published properties of TJPMColorListBox }
          property ItemHeight;
          property Font;
          property TabOrder;
          property TabStop;
          property Visible;
          property OnClick;
          property OnDblClick;
          property OnDragDrop;
          property OnEnter;
          property OnExit;
          property OnKeyDown;
          property OnKeyPress;
          property OnKeyUp;
          property OnMouseDown;
          property OnMouseMove;
          property OnMouseUp;
          {: Color which was selected in the combo-box }
          property SelectColor: TColor read GetSelectColor write SetSelectColor;
          {: Show system colors such as clWIndows }
          property ShowSystemColors: Boolean read GetShowSystemColors
               write SetShowSystemColors
               default True;
          {: Show additional colors such as Pink, Lemon, etc.}
          property ShowAdditionalColors: Boolean read GetShowAdditionalColors write
               SetShowAdditionalColors default True;
          {: Width of the ComboBox}
          property Width default 126;
          {: Show the Custom Color item which pops-up the color selection dialog}
          property ShowCustomColor: Boolean read GetShowCustomColor write SetShowCustomColor default true;
          {: This is width of the color-preview box which displays the color}
          property PreviewBoxWidth: Integer read GetPreviewBoxWidth write SetPreviewBoxWidth default 40;
          {: Show a border around the preview box}
          property ShowBoxBorder: Boolean read GetShowBoxBorder write SetShowBoxBorder;
          {: Show a preview box in ListControl}
          property ShowBox: Boolean read GetShowBox write SetShowBox;
          {: Show the text on using the color as a background}
          property TextOnColorBackground: Boolean read GetTextOnColorBackground write SetTextOnColorBackground;
          {: ShowStyledText variable indicates that we should draw text outside the preview with a particular
           Font or color}
          property ShowStyledText: Boolean read GetShowStyledText write SetShowStyledText;

          {: Displays the Current Version of the Component. }
          property Version: string read GetJPMVersion write SetJPMVersion stored false;
     end;

implementation
uses SysUtils, JPMConst;

{ Method to set variable and property values and create objects }

procedure TJPMColorListBox.AutoInitialize;
begin
     Sorted := False;
     MultiSelect := False;
     Style := lbOwnerDrawFixed;
     FColorSelObject := TJPMColorListObject.Create (Items);
     FColorSelObject.ShowAdditionalColors := True;
     FColorSelObject.ShowSystemColors := True;
     FColorSelObject.ShowCustomColor := True;
     FColorSelObject.ShowBoxWidth := 40;
     Width := 126;
end; { of AutoInitialize }

{ Method to free any objects created by AutoInitialize }

procedure TJPMColorListBox.AutoDestroy;
begin
     FColorSelObject.Free;
end; { of AutoDestroy }

function TJPMColorListBox.GetJPMVersion: string;
begin
     Result := JPMVersion;
end;

procedure TJPMColorListBox.SetJPMVersion (const Value: string);
begin
     // Do nothing
end;

{ Read method for property SelectColor }

function TJPMColorListBox.GetSelectColor: TColor;
begin
     if csReading in ComponentState then
          Result := FColorSelObject.SelectColor
     else if ItemIndex <> -1 then
          Result := TColor (Items.Objects [ItemIndex])
     else
          Result := clBlack;
end;

{ Write method for property SelectColor }

procedure TJPMColorListBox.SetSelectColor (Value: TColor);
begin
     FColorSelObject.SelectColor := Value;
     FColorSelObject.Items.Objects [FColorSelObject.Items.Count - 1] := TObject (Value);

     ItemIndex := Items.IndexOfObject (TObject (Value));
     if ItemIndex < 0 then
          ItemIndex := Items.Count - 1;
end;

{ Read method for property ShowSystemColors }

function TJPMColorListBox.GetShowSystemColors: Boolean;
begin
     Result := FColorSelObject.ShowSystemColors;
end;

{ Write method for property ShowSystemColors }

procedure TJPMColorListBox.SetShowSystemColors (Value: Boolean);
begin
     FColorSelObject.ShowSystemColors := Value;
     {We do not want to create the Color items if we are loading lest
     we create problems.  In any case, it would only duplicate a single action}
     if not (csReading in ComponentState) then
          PopulateComboBox;
end;

{ Override OnClick handler from TESBCustomComboBox }

procedure TJPMColorListBox.Click;
begin
     { Code to execute before activating click
       behavior of component's parent class }

     { Activate click behavior of parent }
     inherited Click;
     {I Wish I could put this in an Change vritual procedure like the ComboBoxes or at least get an LB_SETSEL - oh well
     {if last - show color dialog-box if appropriate}

     if (ItemIndex = (Items.Count - 1)) then
     begin
          ExecuteDialog; //show color dialog-box
          Repaint; //we have to redraw the listbox to get this to work
     end
     else if ItemIndex >= 0 then
          FColorSelObject.SelectColor := TColor (Items.Objects [ItemIndex]); //reflect the color in SelectColor
     { Code to execute after click behavior
       of parent }
end;

{ Override OnKeyPress handler from TESBCustomComboBox }

procedure TJPMColorListBox.KeyPress (var Key: Char);
const
     TabKey = Char (VK_TAB);
     EnterKey = Char (VK_RETURN);
begin
     { Key contains the character produced by the keypress.
      It can be tested or assigned a new value before the
      call to the inherited KeyPress method.  Setting Key
      to #0 before call to the inherited KeyPress method
      terminates any further processing of the character. }

     { Activate KeyPress behavior of parent }
     inherited KeyPress (Key);

     { Code to execute after KeyPress behavior of parent }
end;

constructor TJPMColorListBox.Create (AOwner: TComponent);
begin
     { Call the Create method of the parent class }
     inherited Create (AOwner);

     { AutoInitialize sets the initial values of variables and      }
     { properties; also, it creates objects for properties of       }
     { standard Delphi object types (e.g., TFont, TTimer,           }
     { TPicture) and for any variables marked as objects.           }
     { AutoInitialize method is generated by Component Create.      }
     AutoInitialize;
     { Code to perform other tasks when the component is created }
end;

destructor TJPMColorListBox.Destroy;
begin
     { AutoDestroy, which is generated by Component Create, frees any   }
     { objects created by AutoInitialize.                               }
     AutoDestroy;

     { Here, free any other dynamic objects that the component methods  }
     { created but have not yet freed.  Also perform any other clean-up }
     { operations needed before the component is destroyed.             }

     { Last, free the component by calling the Destroy method of the    }
     { parent class.                                                    }
     inherited Destroy;
end;

{ //override to display colors in box }

procedure TJPMColorListBox.DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
     if (odFocused in State) or (odSelected in State) then
     begin
          Canvas.Font.Color := clHighLightText;
          Canvas.Brush.Color := clHighLight;
     end //if (odFocused in State) or (odSelected in State) then
     else
     begin
          Canvas.Font.Color := Font.Color;
          Canvas.Brush.Color := Color;
     end; //else ... if (odFocused in State) or (odSelected in State) then
     Canvas.FillRect (Rect);
     {Color preview rectange}
     FColorSelObject.DrawBox (Canvas, Index, Rect, State);
     inherited DrawItem (Index, Rect, State);

end;

{ //Set Up color selection for combo-box }

procedure TJPMColorListBox.Loaded;
begin
     inherited Loaded;
     PopulateComboBox;
     {Now that we have loaded all items, we set the selection.  If the loaded color
     is NOT in the selection and ShowCustomColors is True, we select the CustomColors
     entry and set its color to the property value}
   {  if Items.IndexOfObject(TObject(FColorSelObject.SelectColor)) <> -1 then
        ItemIndex := Items.IndexOfObject(TObject(FColorSelObject.SelectColor))
     else
       if FColorSelObject.ShowCustomColor then
       begin
         ItemIndex := Items.Count - 1;
         Items.Objects[Items.Count - 1] := TObject(FColorSelObject.SelectColor);
       end
       else
         ItemIndex := 0;   }
end;

//this loads the colors into the listbox

procedure TJPMColorListBox.PopulateComboBox;
begin
     FColorSelObject.PopulateComboBox;
     {Now that we have loaded all items, we set the selection.  If the loaded color
     is NOT in the selection and ShowCustomColors is True, we select the CustomColors
     entry and set its color to the property value}
     if Items.IndexOfObject (TObject (FColorSelObject.SelectColor)) <> -1 then
          ItemIndex := Items.IndexOfObject (TObject (FColorSelObject.SelectColor))
     else if FColorSelObject.ShowCustomColor then
     begin
          ItemIndex := Items.Count - 1;
          Items.Objects [Items.Count - 1] := TObject (FColorSelObject.SelectColor);
     end
     else
          ItemIndex := 0;
     {We have to redraw the control so it is updated}
     Repaint;
end;

procedure TJPMColorListBox.SetShowCustomColor (Value: Boolean);
begin
     FColorSelObject.ShowCustomColor := Value;
     if not (csReading in ComponentState) then
          PopulateComboBox; //repopulate list
end;

{Write method for property ShowAdditionalColors }

procedure TJPMColorListBox.SetShowAdditionalColors (Value: Boolean);
begin
     FColorSelObject.ShowAdditionalColors := Value;
     if not (csReading in ComponentState) then
          PopulateComboBox; //repopulate list
end;

function TJPMColorListBox.ExecuteDialog: Boolean;
{The key assumption is that "Custom Color" is the last item in the list}
begin
     Result := False; {assume false for expediancy}
     if FColorSelObject.ExecDialogBoxEnh (JPMCustomColors) then
     begin
          ItemIndex := Items.Count - 1 {Select "Custom Color" for user if this was called independantly of the combobox}
     end
     else
          FColorSelObject.SelectColor := TColor (Items.Objects [ItemIndex]); //make sure that the original color is preserved
end;

function TJPMColorListBox.GetShowAdditionalColors: Boolean;
begin
     Result := FColorSelObject.ShowAdditionalColors;
end;

function TJPMColorListBox.GetShowCustomColor: Boolean;
begin
     Result := FColorSelObject.ShowCustomColor;
end;

function TJPMColorListBox.GetPreviewBoxWidth: Integer;
begin
     Result := FColorSelObject.ShowBoxWidth;
end;

procedure TJPMColorListBox.SetPreviewBoxWidth (Value: Integer);
begin
     FColorSelObject.ShowBoxWidth := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

procedure TJPMColorListBox.SetParent (AParent: TWinControl);
begin
     inherited;
     if AParent <> nil then
          PopulateComboBox;
end;

function TJPMColorListBox.GetShowBox: Boolean;
begin
     Result := FColorSelObject.ShowBox;

end;

function TJPMColorListBox.GetShowBoxBorder: Boolean;
begin
     Result := FColorSelObject.ShowBoxBorder;
end;

procedure TJPMColorListBox.SetShowBox (const Value: Boolean);
begin
     FColorSelObject.ShowBox := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

procedure TJPMColorListBox.SetShowBoxBorder (const Value: Boolean);
begin
     FColorSelObject.ShowBoxBorder := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

function TJPMColorListBox.GetTextOnColorBackground: Boolean;
begin
     Result := FColorSelObject.TextOnColorBackground;
end;

procedure TJPMColorListBox.SetTextOnColorBackground (const Value: Boolean);
begin
     FColorSelObject.TextOnColorBackground := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

function TJPMColorListBox.GetShowStyledText: Boolean;
begin
     Result := FColorSelObject.ShowStyledText;
end;

procedure TJPMColorListBox.SetShowStyledText (const Value: Boolean);
begin
     FColorSelObject.ShowStyledText := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

end.
