
{: This unit contains the TJPMEColorComboBox which is a color selection combo-box
 which is dependant on ESBPCS.
}
{Release notes
  1/24/2000 - Jan -created the first version
  1/26/2000 - Renamed component  TJPMEColorComboBox to signify that this is an ESBPCS
       descendant
       Fixed a bug where the color list was not updated when you set the
         ShowSystemColors properties at run-time
       Cleaned-up source a little bit
       added ShowCustomColor and DisplayColors properties
       When ShowCustomColor is true, an entry appears in the combo-box with "Custom Colors"
        which when selected brings up the color selection dialog-box and when Ok is pressed,
        the custom color is displayed if DisplayColors is true and SelectColor will reflect
        the custom color.
  1/27/2000 added some additional non-standard colors at the advice of Glenn Crouch.  These colors
        were defined by Jim Gunkel, Nevrona and are used in good faith that their use is free.
        Moved the Color Selection Dialog-box code to ExecuteDialog so that they could be called
        from a button or other control.
  1/28/2000 fixed a bug wher color selection was not reflected in the SelectColor property
  1/29/2000 made some functionality and properties in a contained object (FColorSelObject).
  I am hoping by doing so that much of the functionality can be placed in unrelated controls such as ListBoxes
            made a few more improvements to the custom color feature so a user can select any color
  1/31/2000 published inherited properties ItemHeight and Font
            moved component to "JPM Open" tab
  2-2-2000 fixed problem with dynamically created controls
  }
unit JPMEColorComboBox;

interface

{$I JPM.INC}

uses Classes, Controls, Dialogs,
     Esbpcscombobox, Graphics,
     JPMColorListObject, Stdctrls, Windows;

type
     TJPMEColorComboBox = class (TESBCustomComboBox)
     private
          { Private fields of TJPMEColorComboBox }
          FColorSelObject: TJPMColorListObject;
          { Used to prevent Dialog Appearing }
          FHideDialog: Boolean; //esb
          {: Method to set variable and property values and create objects }
          procedure AutoInitialize;
          {: Method to free any objects created by AutoInitialize }
          procedure AutoDestroy;
          {: Read method for property SelectColor }
          function GetSelectColor: TColor;
          {: Write method for property SelectColor }
          procedure SetSelectColor (Value: TColor);
          {: Read method for property ShowSystemColors }
          function GetShowSystemColors: Boolean;
          {: Write method for property AdditionalColors}
          procedure SetShowAdditionalColors (Value: Boolean);
          {: Read method for property AdditionalColors}
          function GetShowAdditionalColors: Boolean;
          {: Write method for property ShowSystemColors }
          procedure SetShowSystemColors (Value: Boolean);
          {: Write method for property ShowCustomColor }
          procedure SetShowCustomColor (Value: Boolean);
          {: Read method for property ShowCustomColor }
          function GetShowCustomColor: Boolean;
          {: PreviewBoxWidth write method for PreviewBoxWidth }
          procedure SetPreviewBoxWidth (Value: Integer);
          {: PreviewBoxWidth read method for PreviewBoxWidth }
          function GetPreviewBoxWidth: Integer;
          function GetShowBox: Boolean;
          function GetShowBoxBorder: Boolean;
          function GetShowStyledText: Boolean;
          function GetTextOnColorBackground: Boolean;
          procedure SetShowBox (const Value: Boolean);
          procedure SetShowBoxBorder (const Value: Boolean);
          procedure SetShowStyledText (const Value: Boolean);
          procedure SetTextOnColorBackground (const Value: Boolean);

     protected
          { Protected methods of TJPMEColorComboBox }
          procedure SetParent (AParent: TWinControl); override;
          procedure Change; override;
          procedure Click; override;
          procedure KeyPress (var Key: Char); override;
          {: Set Up color selection for combo-box }
          procedure Loaded; override;
          procedure PopulateComboBox; virtual;
          function GetJPMVersion: string;
          procedure SetJPMVersion (const Value: string);
     public
          { Public fields and properties of TJPMColorComboBox }
          {: Creates the comobobox}
          constructor Create (AOwner: TComponent); override;
          {: Destroys the combobox}

          destructor Destroy; override;
          {: override to display colors in box }
          procedure DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
          {: method for calling ColorSelection Dialog-box }
          function ExecuteDialog: Boolean;
     published
          { Published properties of TJPMColorComboBox }
          property ItemHeight;
          property Font;
          property TabStop;
          property TabOrder;
          property Visible;
          property OnChange;
          property OnClick;
          property OnDblClick;
          property OnDragDrop;
          property OnEnter;
          property OnExit;
          property OnKeyDown;
          property OnKeyPress;
          property OnKeyUp;
          property OnMouseDown;
          property OnMouseMove;
          property OnMouseUp;
          {: Color which was selected in the combo-box }
          property SelectColor: TColor read GetSelectColor write SetSelectColor
               stored ValueStored;
          {: Show system colors such as clWindows }
          property ShowSystemColors: Boolean read GetShowSystemColors
               write SetShowSystemColors
               default True;
          {: Show additional colors such as Pink, Lemon, etc.}
          property ShowAdditionalColors: Boolean read GetShowAdditionalColors write
               SetShowAdditionalColors default True;
          {: Width of the combobox}
          property Width default 126;
          {: Show the Custom Color item which pops-up the color selection dialog}
          property ShowCustomColor: Boolean read GetShowCustomColor write SetShowCustomColor default true;
          {: This is width of the color-preview box which displays the color}
          property PreviewBoxWidth: Integer read GetPreviewBoxWidth write SetPreviewBoxWidth default 40;
          {: Show a border around the preview box}
          property ShowBoxBorder: Boolean read GetShowBoxBorder write SetShowBoxBorder;
          {: Show a preview box in ListControl}
          property ShowBox: Boolean read GetShowBox write SetShowBox;
          {: Show the text on using the color as a background}
          property TextOnColorBackground: Boolean read GetTextOnColorBackground write SetTextOnColorBackground;
          {: ShowStyledText variable indicates that we should draw text outside the preview with a particular
           Font or color}
          property ShowStyledText: Boolean read GetShowStyledText write SetShowStyledText;
          {: Displays the Current Version of the Component. }
          property Version: string read GetJPMVersion write SetJPMVersion stored false;
     end;

implementation
uses JPMConst, SysUtils;

{ Method to set variable and property values and create objects }

procedure TJPMEColorComboBox.AutoInitialize;
begin
     FColorSelObject := TJPMColorListObject.Create (Items);
     FColorSelObject.ShowSystemColors := True;
     FColorSelObject.ShowAdditionalColors := True;
     FColorSelObject.ShowSystemColors := True;
     FColorSelObject.ShowCustomColor := True;
     FColorSelObject.ShowBoxWidth := 40;
     Width := 126;
end; { of AutoInitialize }

{ Method to free any objects created by AutoInitialize }

procedure TJPMEColorComboBox.AutoDestroy;
begin
     FColorSelObject.Free;
end; { of AutoDestroy }

function TJPMEColorComboBox.GetJPMVersion: string;
begin
     Result := JPMVersion;
end;

procedure TJPMEColorComboBox.SetJPMVersion (const Value: string);
begin
     // Do nothing
end;

{ Read method for property SelectColor }

function TJPMEColorComboBox.GetSelectColor: TColor;
begin
     Result := FColorSelObject.SelectColor;
end;

{ Write method for property SelectColor }

procedure TJPMEColorComboBox.SetSelectColor (Value: TColor);
begin
     FColorSelObject.SelectColor := Value;
     FColorSelObject.Items.Objects [FColorSelObject.Items.Count - 1] := TObject (Value); //esb

     FHideDialog := True; //esb
     try
          ItemIndex := Items.IndexOfObject (TObject (Value));
          if ItemIndex < 0 then //esb
               ItemIndex := Items.Count - 1;
     finally
          FHideDialog := False; //esb
     end;
end;

{ Read method for property ShowSystemColors }

function TJPMEColorComboBox.GetShowSystemColors: Boolean;
begin
     Result := FColorSelObject.ShowSystemColors;
end;

{ Write method for property ShowSystemColors }

procedure TJPMEColorComboBox.SetShowSystemColors (Value: Boolean);
begin
     FColorSelObject.ShowSystemColors := Value;
     {We do not want to create the Color items if we are loading lest
     we create problems.  In any case, it would only duplicate a single action}
     if not (csReading in ComponentState) then
          PopulateComboBox;
end;

{ Override OnChange handler from TESBCustomComboBox }

procedure TJPMEColorComboBox.Change;
begin
     {if last - show color dialog-box if appropriate}
     if (ItemIndex = (Items.Count - 1)) then
          ExecuteDialog //show color dialog-box
     else if ItemIndex >= 0 then //esb
          FColorSelObject.SelectColor := TColor (Items.Objects [ItemIndex]); //reflect the color in SelectColor
     inherited Change;
end;

{ Override OnClick handler from TESBCustomComboBox }

procedure TJPMEColorComboBox.Click;
begin
     { Code to execute before activating click
       behavior of component's parent class }

     { Activate click behavior of parent }
     inherited Click;

     { Code to execute after click behavior
       of parent }
end;

{ Override OnKeyPress handler from TESBCustomComboBox }

procedure TJPMEColorComboBox.KeyPress (var Key: Char);
const
     TabKey = Char (VK_TAB);
     EnterKey = Char (VK_RETURN);
begin
     { Key contains the character produced by the keypress.
      It can be tested or assigned a new value before the
      call to the inherited KeyPress method.  Setting Key
      to #0 before call to the inherited KeyPress method
      terminates any further processing of the character. }

     { Activate KeyPress behavior of parent }
     inherited KeyPress (Key);

     { Code to execute after KeyPress behavior of parent }
end;

constructor TJPMEColorComboBox.Create (AOwner: TComponent);
begin
     { Call the Create method of the parent class }
     inherited Create (AOwner);

     { AutoInitialize sets the initial values of variables and      }
     { properties; also, it creates objects for properties of       }
     { standard Delphi object types (e.g., TFont, TTimer,           }
     { TPicture) and for any variables marked as objects.           }
     { AutoInitialize method is generated by Component Create.      }
     AutoInitialize;
     { Code to perform other tasks when the component is created }
end;

destructor TJPMEColorComboBox.Destroy;
begin
     { AutoDestroy, which is generated by Component Create, frees any   }
     { objects created by AutoInitialize.                               }
     AutoDestroy;

     { Here, free any other dynamic objects that the component methods  }
     { created but have not yet freed.  Also perform any other clean-up }
     { operations needed before the component is destroyed.             }

     { Last, free the component by calling the Destroy method of the    }
     { parent class.                                                    }
     inherited Destroy;
end;

{ //override to display colors in box }

procedure TJPMEColorComboBox.DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState);

begin
     if FColorSelObject.ShowBoxWidth < 1 then {if we are not to display the colors in the combo-box, simply call inherited DrawItem}
          inherited DrawItem (Index, Rect, State)
     else //if not FDisplayColors then
     begin
          {This part was taken from ESBPCS and used with permission.  The code basically provides
          a more consistant feel with ESBPCS.  I know that this is duplicated in the inherited call but
          I also will not be using that to draw the complete item.  I only use that to draw the rest
          of the combo-box.  Therefore, this is necessary.}
          if (odFocused in State) or (odSelected in State) then
          begin
               Canvas.Font.Color := ColorSet.ListHighlightFontColor;
               Canvas.Brush.Color := ColorSet.ListHighlightColor;
          end //if (odFocused in State) or (odSelected in State) then
          else
          begin
               if ReadOnly then
               begin
                    Canvas.Brush.Color := ColorSet.ROColor;
                    Canvas.Font.Color := Font.Color;
               end //if ReadOnly then
               else
               begin
                    Canvas.Font.Color := ColorSet.ListFontColor;
                    Canvas.Brush.Color := ColorSet.ListColor;
               end; //else ... if ReadOnly then
          end; //else ... if (odFocused in State) or (odSelected in State) then
          Canvas.FillRect (Rect);
          {Color preview rectange}
          FColorSelObject.DrawBox (Canvas, Index, Rect, State);
          inherited DrawItem (Index, Rect, State);
     end; //if not FDisplayColors then
end;

{ //Set Up color selection for combo-box }

procedure TJPMEColorComboBox.Loaded;
begin
     inherited Loaded;
     PopulateComboBox;
end;

//this loads the colors into the listbox

procedure TJPMEColorComboBox.PopulateComboBox;
begin
     FColorSelObject.PopulateComboBox;
     ItemIndex := Items.IndexOfObject (TObject (FColorSelObject.SelectColor));
end;

procedure TJPMEColorComboBox.SetShowCustomColor (Value: Boolean);
begin
     FColorSelObject.ShowCustomColor := Value;
     if not (csReading in ComponentState) then
          PopulateComboBox; //repopulate list
end;

{Write method for property ShowAdditionalColors }

procedure TJPMEColorComboBox.SetShowAdditionalColors (Value: Boolean);
begin
     FColorSelObject.ShowAdditionalColors := Value;
     if not (csReading in ComponentState) then
          PopulateComboBox; //repopulate list
end;

function TJPMEColorComboBox.ExecuteDialog: Boolean;
{The key assumption is that "Custom Color" is the last item in the list}
begin
     Result := False; {assume false for expediancy}
     if not ReadOnly and not FHideDialog and FColorSelObject.ExecDialogBoxEnh (JPMCustomColors) then
          ItemIndex := Items.Count - 1 {Select "Custom Color" for user if this was called independantly of the combobox}
     else
          FColorSelObject.SelectColor := TColor (Items.Objects [ItemIndex]); //make sure that the original color is preserved
end;

function TJPMEColorComboBox.GetShowAdditionalColors: Boolean;
begin
     Result := FColorSelObject.ShowAdditionalColors;
end;

function TJPMEColorComboBox.GetShowCustomColor: Boolean;
begin
     Result := FColorSelObject.ShowCustomColor;
end;

function TJPMEColorComboBox.GetPreviewBoxWidth: Integer;
begin
     Result := FColorSelObject.ShowBoxWidth;
end;

procedure TJPMEColorComboBox.SetPreviewBoxWidth (Value: Integer);
begin
     FColorSelObject.ShowBoxWidth := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is changed
end;

procedure TJPMEColorComboBox.SetParent (AParent: TWinControl);
var
     Populate: Boolean;
     {I know that this looks redundant but it turns out that when loading, SetParent is called
      twice with the same parent and also during loading.  We only want to populate once}
begin
     Populate := (AParent <> nil) and (AParent <> Parent) and (not (csReading in ComponentState)) and
          (not (csLoading in ComponentState));
     inherited;
     if Populate then
          PopulateComboBox;
end;

function TJPMEColorComboBox.GetShowBox: Boolean;
begin
     Result := FColorSelObject.ShowBox;
end;

function TJPMEColorComboBox.GetShowBoxBorder: Boolean;
begin
     Result := FColorSelObject.ShowBoxBorder;
end;

function TJPMEColorComboBox.GetShowStyledText: Boolean;
begin
     Result := FColorSelObject.ShowStyledText;
end;

function TJPMEColorComboBox.GetTextOnColorBackground: Boolean;
begin
     Result := FColorSelObject.TextOnColorBackground;
end;

procedure TJPMEColorComboBox.SetShowBox (const Value: Boolean);
begin
     FColorSelObject.ShowBox := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

procedure TJPMEColorComboBox.SetShowBoxBorder (const Value: Boolean);
begin
     FColorSelObject.ShowBoxBorder := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

procedure TJPMEColorComboBox.SetShowStyledText (const Value: Boolean);
begin
     FColorSelObject.ShowStyledText := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

procedure TJPMEColorComboBox.SetTextOnColorBackground (
     const Value: Boolean);
begin
     FColorSelObject.TextOnColorBackground := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

end.
